import { APIResource } from "../../resource.js";
import * as PromptCachingAPI from "./prompt-caching/prompt-caching.js";
export declare class Beta extends APIResource {
    promptCaching: PromptCachingAPI.PromptCaching;
}
export declare namespace Beta {
    export import PromptCaching = PromptCachingAPI.PromptCaching;
}
//# sourceMappingURL=beta.d.ts.map